import cv2

def guess_pos(img, pos):
    x = 0
    y = 0
    align = ''
    if 'bottom' in pos:
        y = img.shape[0]
        align = 'bottom '
    if 'right' in pos:
        x = img.shape[1]
        align += 'right'
    return (x, y), align

def write(
    img,
    text,
    font=cv2.FONT_HERSHEY_SIMPLEX,
    pos=(0,0),
    font_scale=1,
    font_thickness=2,
    color=(255, 255, 255),
    bg_color=None,
    align="",
):
    text = str(text)
    if isinstance(pos, str):
        pos, align = guess_pos(img, pos)

    x, y = pos
    (text_w, text_h), _ = cv2.getTextSize(text, font, font_scale, font_thickness)
    if 'bottom' in align:
        y -= text_h
    if 'right' in align:
        x -= text_w

    if bg_color is not None:
        cv2.rectangle(
            img,
            (x, y),
            (x + text_w, y + text_h),
            bg_color,
            -1
        )
    cv2.putText(
        img,
        text,
        (x, int(y + text_h + font_scale - 1)),
        font,
        font_scale,
        color,
        font_thickness
    )

    return (text_w, text_h)


import cv2
from imutils.object_detection import non_max_suppression
import numpy as np

def match(image, template, threshold=0.9, mask=None):
    result = cv2.matchTemplate(image, template, cv2.TM_CCOEFF_NORMED, mask=mask)

    (yCoords, xCoords) = np.where(result >= threshold)
    (tH, tW) = template.shape[:2]

    coords = [(x, y, x + tW, y + tH) for (x, y) in zip(xCoords, yCoords)]
    pick = non_max_suppression(np.array(coords))
    return pick


def union(image, template):
    """
    Extract only the pixels that match the template.
    """
    diff = cv2.absdiff(image, template)
    mask = cv2.cvtColor(diff, cv2.COLOR_BGR2GRAY)
    imask = mask == 0

    canvas = np.zeros_like(image, np.uint8)
    canvas[imask] = image[imask]
    return canvas


import cv2
import random

special_keys = {
    65362: 'up',
    65364: 'down',
    65361: 'left',
    65363: 'right',
    7: 'esc',
    32: 'space',
    13: 'enter',
}

def wait_key(max_time=60000, default=None, delay = 500):
    time = 0
    while time < max_time:
        pressed = cv2.waitKeyEx(delay)
        if pressed in special_keys:
            return special_keys[pressed]
        if pressed != -1:
            return chr(pressed)
        time += delay
    return default


def get_scaled_roi(image, scale):
    w = image.shape[1] * scale
    h = image.shape[0] * scale
    name = f'ROI_{random.random()}'
    bounds = cv2.selectROI(name, cv2.resize(image, (w, h)))
    cv2.destroyWindow(name)
    return [int(i/scale) for i in bounds]
